#include "videowidget.hpp"

#include <string>

VideoWidget::VideoWidget(QWidget *parent) :

#if defined(Q_WS_MAC)
    QMacCocoaViewContainer(NULL, parent),
#else
    QWidget(parent),
#endif

    m_instance(NULL),
    m_mediaPlayer(NULL)
{
    static const char * const args[] = {
        "-I", "dummy", /* Don't use any interface */
        //"--verbose=-1", "--quiet",
        "--no-video-title-show",
        "--ignore-config", /* Don't use VLC's config */

#if defined(Q_WS_MAC)
        "--vout=macosx",
#endif

        "--plugin-path=plugins" };

    m_instance = libvlc_new(sizeof(args) / sizeof(char*), args);

    m_mediaPlayer = libvlc_media_player_new(m_instance);
    registerFinishedCallback();

#if defined(Q_WS_WIN)
     libvlc_media_player_set_hwnd(m_mediaPlayer, winId());
#elif defined(Q_WS_X11)
    libvlc_media_player_set_hwnd(m_mediaPlayer,
                                 reinterpret_cast<void *>(winId()));
    libvlc_media_player_set_xwindow(m_mediaPlayer, winId());
#elif defined(Q_WS_MAC)
    createCocoaView();
    libvlc_media_player_set_nsobject(m_mediaPlayer, cocoaView());
#endif
}

VideoWidget::~VideoWidget()
{
    unregisterFinishedCallback();
    stop();

#if defined(Q_WS_MAC)
    libvlc_media_player_set_nsobject(m_mediaPlayer, NULL);
#endif

    libvlc_media_player_release(m_mediaPlayer);
    libvlc_release(m_instance);
}

void VideoWidget::play(const QString &url)
{
    libvlc_media_t *media = NULL;

#if defined(Q_OS_WIN32)
    QString backslashUrl(url);
    backslashUrl.replace("/", "\\");
    media = libvlc_media_new_path(m_instance, backslashUrl.toStdString().c_str());
#else
    media = libvlc_media_new_path(m_instance, url.toStdString().c_str());
#endif

    if (!media) return;

    libvlc_media_player_set_media(m_mediaPlayer, media);
    libvlc_media_release(media);
    libvlc_media_player_play(m_mediaPlayer);
}

void VideoWidget::stop()
{
    if (libvlc_media_player_is_playing(m_mediaPlayer))
    {
        libvlc_media_player_stop(m_mediaPlayer);
    }
}

void VideoWidget::finishedCallback(const libvlc_event_t *event,
                                            void *obj)
{
    if (event->type == libvlc_MediaPlayerEndReached)
    {
        static_cast<VideoWidget*>(obj)->emitFinished();
    }
}

void VideoWidget::registerFinishedCallback()
{
    libvlc_event_attach(libvlc_media_player_event_manager(m_mediaPlayer),
                        libvlc_MediaPlayerEndReached,
                        &finishedCallback,
                        this);
}

void VideoWidget::unregisterFinishedCallback()
{
    libvlc_event_detach(libvlc_media_player_event_manager(m_mediaPlayer),
                        libvlc_MediaPlayerEndReached,
                        &finishedCallback,
                        this);
}

void VideoWidget::emitFinished()
{
    emit finished();
}
